/*******************************************************************************
 *
 * FILENAME:     Gaussian Noise.cpp
 * DESCRIPTION:  generate gaussian noise using boost library
 *
 * AUTHOR:       Jun Liu (jun@cs.man.ac.uk)
 * INSTITUTION:  Advanced Interfaces Group,
 *               Department of Computer Science,
 *               the University of Manchester, United Kingdom
 * DATE:         August 2006
 *
 *******************************************************************************/

#include "GaussianNoise.h"
#include <ctime>

mt19937 GaussianNoise::m_rng;

void GaussianNoise::init()
{
        // select random number generator
        // seed generator with #seconds since 1970
    m_rng.seed(std::time(0));
}

double GaussianNoise::generateGaussianNoise(double mean, double variance)
{
        // select desired probability distribution
    normal_distribution<double> norm_dist(mean, variance);
    
        // bind random number generator to distribution, forming a function
    variate_generator<mt19937&, normal_distribution<double> >  normal_sampler(m_rng, norm_dist);
    
        // sample from the distribution
    return normal_sampler();
}
