/*******************************************************************************
 *
 * FILENAME:     RotationMatrix.cpp
 * DESCRIPTION:  rotation matrix
 *
 * AUTHOR:       Jun Liu (jun@cs.man.ac.uk)
 * INSTITUTION:  Advanced Interfaces Group,
 *               Department of Computer Science,
 *               the University of Manchester, United Kingdom
 * DATE:         July 2006
 *
 *******************************************************************************/

#include "RotationMatrix.h"
#include <cmath>
#include <vnl/algo/vnl_matrix_inverse.h>
#include <iostream>
using namespace std;


vnl_double_3x3 RotationMatrix::fromEulerAngles(double roll, double pitch, double yaw)
{

        // convert first to quaternion
    
    double cyaw, cpitch, croll, syaw, spitch, sroll;
    double cyawcpitch, syawspitch, cyawspitch, syawcpitch;
    double n, x, y, z;

    cyaw = cos(0.5 * yaw);
    cpitch = cos(0.5 * pitch);
    croll = cos(0.5 * roll);
    syaw = sin(0.5 * yaw);
    spitch = sin(0.5 * pitch);
    sroll = sin(0.5 * roll);
    
    cyawcpitch = cyaw * cpitch;
    syawspitch = syaw * spitch;
    cyawspitch = cyaw * spitch;
    syawcpitch = syaw * cpitch;
    
    n = cyawcpitch * croll + syawspitch * sroll;
    x = cyawcpitch * sroll - syawspitch * croll;
    y = cyawspitch * croll + syawcpitch * sroll;
    z = syawcpitch * croll - cyawspitch * sroll;

    double length = sqrt( n*n + x*x + y*y + z*z);
    n /= length;
    x /= length;
    y /= length;
    z /= length;
    
    double q00 = n * n;
    double q11 = x * x;
    double q22 = y * y;
    double q33 = z * z;

    vnl_double_3x3 rv;
    rv(0,0) = q00 + q11 - q22 - q33;
    rv(0,1) = 2.0 * (x * y - n * z);
    rv(0,2) = 2.0 * (x * z + n * y);
    rv(1,0) = 2.0 * (x * y + n * z);
    rv(1,1) = q00 - q11 + q22 - q33;
    rv(1,2) = 2.0 * (y * z - n * x);
    rv(2,0) = 2.0 * (x * z - n * y);
    rv(2,1) = 2.0 * (y * z + n * x);
    rv(2,2) = q00 - q11 - q22 + q33;

    
        // test
        // vnl_matrix<double> matrix (rv);
        // cout << matrix.transpose() << endl;
        // matrix = vnl_matrix_inverse<double> (matrix);
        // cout << matrix << endl;
    


    return rv;
}

vnl_double_3x3 RotationMatrix::getIdentity()
{
    vnl_double_3x3 rv;
    rv(0,0) = 1.0; rv(0,1) = 0.0; rv(0,2) = 0.0;
    rv(1,0) = 0.0; rv(1,1) = 1.0; rv(1,2) = 0.0;
    rv(2,0) = 0.0; rv(2,1) = 0.0; rv(2,2) = 1.0;
    return rv;
}
